@@ -1,28 +1,3 @@
-/*
-  Racket
-  Copyright (c) 2004-2018 PLT Design Inc.
-  Copyright (c) 1995-2001 Matthew Flatt
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301 USA.
-
-  libscheme
-  Copyright (c) 1994 Brent Benson
-  All rights reserved.
-*/
-
 #include "schpriv.h"
 #include "schminc.h"
 #include "schmach.h"
@@ -347,9 +322,10 @@ static void init_startup_env(void)
   
 #if USE_COMPILED_STARTUP
   if (builtin_ref_counter != EXPECTED_PRIM_COUNT) {
-    printf("Primitive count %d doesn't match expected count %d\n"
-	   "Turn off USE_COMPILED_STARTUP in src/schminc.h\n",
-	   builtin_ref_counter, EXPECTED_PRIM_COUNT);
+    fprintf(stderr,
+            "Primitive count %d doesn't match expected count %d\n"
+            "Update the count in src/schminc.h and bump the version in src/schvers.h\n",
+            builtin_ref_counter, EXPECTED_PRIM_COUNT);
     abort();
   }
 #endif
