@@ -1,28 +1,3 @@
-/*
-  Racket
-  Copyright (c) 2004-2018 PLT Design Inc.
-  Copyright (c) 1995-2001 Matthew Flatt
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301 USA.
-
-  libscheme
-  Copyright (c) 1994 Brent Benson
-  All rights reserved.
-*/
-
 /* This file implements a bytecode pass to insert hook that trigger
    JIT compilation. This pass is performed after bytecode is marshaled
    or unmarshaled.
@@ -33,8 +8,19 @@
 #include "schpriv.h"
 #include "schrunst.h"
 
+THREAD_LOCAL_DECL(static Scheme_Object *current_linklet_native_lambdas);
+static int force_jit;
+
 #ifdef MZ_USE_JIT
 
+void scheme_init_jitprep()
+{
+  REGISTER_SO(current_linklet_native_lambdas);
+
+  if (getenv("PLT_EAGER_JIT"))
+    force_jit = 1;
+}
+
 static Scheme_Object *jit_expr(Scheme_Object *expr);
 
 static Scheme_Object *jit_application(Scheme_Object *o)
@@ -434,6 +420,8 @@ Scheme_Object *scheme_case_lambda_jit(Scheme_Object *expr)
       ((Scheme_Lambda *)val)->name = name;
       if (((Scheme_Lambda *)val)->closure_size)
 	all_closed = 0;
+      if (current_linklet_native_lambdas)
+        current_linklet_native_lambdas = scheme_make_pair(val, current_linklet_native_lambdas);
     }
 
     /* Generating the code may cause empty closures to be formed: */
@@ -561,6 +549,10 @@ Scheme_Object *scheme_jit_closure(Scheme_Object *code, Scheme_Object *context)
     ndata = scheme_generate_lambda(data2, 1, NULL);
     data2->u.native_code = ndata;
 
+    if (current_linklet_native_lambdas)
+      current_linklet_native_lambdas = scheme_make_pair((Scheme_Object *)ndata,
+                                                        current_linklet_native_lambdas);
+
     if (!context)
       data->u.jit_clone = data2;
   }
@@ -664,6 +656,9 @@ Scheme_Linklet *scheme_jit_linklet(Scheme_Linklet *linklet, int step)
     return new_linklet;
   }
 
+  if (force_jit)
+    current_linklet_native_lambdas = scheme_null;
+
   i = SCHEME_VEC_SIZE(linklet->bodies);
   bodies = scheme_make_vector(i, NULL);
   while (i--) {
@@ -675,6 +670,9 @@ Scheme_Linklet *scheme_jit_linklet(Scheme_Linklet *linklet, int step)
 
   new_linklet->jit_ready = 2;
 
+  new_linklet->native_lambdas = current_linklet_native_lambdas;
+  current_linklet_native_lambdas = NULL;
+
   return new_linklet;
 }
 
