@@ -613,6 +613,10 @@
 #define scheme_module_bucket (scheme_extension_table->scheme_module_bucket)
 #define scheme_builtin_value (scheme_extension_table->scheme_builtin_value)
 #define scheme_set_global_bucket (scheme_extension_table->scheme_set_global_bucket)
+#define scheme_primitive_module (scheme_extension_table->scheme_primitive_module)
+#define scheme_finish_primitive_module (scheme_extension_table->scheme_finish_primitive_module)
+#define scheme_set_primitive_module_phaseless (scheme_extension_table->scheme_set_primitive_module_phaseless)
+#define scheme_protect_primitive_provide (scheme_extension_table->scheme_protect_primitive_provide)
 #define scheme_make_modidx (scheme_extension_table->scheme_make_modidx)
 #define scheme_dynamic_require (scheme_extension_table->scheme_dynamic_require)
 #define scheme_dynamic_require_reader (scheme_extension_table->scheme_dynamic_require_reader)
