@@ -1,28 +1,3 @@
-/*
-  Racket
-  Copyright (c) 2004-2018 PLT Design Inc.
-  Copyright (c) 1995-2001 Matthew Flatt
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301 USA.
-
-  libscheme
-  Copyright (c) 1994 Brent Benson
-  All rights reserved.
-*/
-
 /* Some copilers don't like re-def of GC_malloc in schemef.h: */
 #ifndef MZ_PRECISE_GC
 # define SCHEME_NO_GC_PROTO
@@ -348,7 +323,7 @@ scheme_init_symbol (Scheme_Startup_Env *env)
                                                             | SCHEME_PRIM_PRODUCES_BOOL);
   scheme_addto_prim_instance("symbol-interned?", p, env);
 
-  ADD_FOLDING_PRIM("symbol<?",                 symbol_lt,                       2, -1, 1, env);  
+  ADD_FOLDING_PRIM("symbol<?",                 symbol_lt,                       1, -1, 1, env);
 
   p = scheme_make_folding_prim(string_to_symbol_prim, "string->symbol", 1, 1, 1);
   SCHEME_PRIM_PROC_FLAGS(p) |= scheme_intern_prim_opt_flags(SCHEME_PRIM_AD_HOC_OPT);
@@ -369,7 +344,7 @@ scheme_init_symbol (Scheme_Startup_Env *env)
   scheme_keyword_p_proc = p;
   scheme_addto_prim_instance("keyword?", p, env);
 
-  ADD_FOLDING_PRIM("keyword<?",                keyword_lt,                       2, -1, 1, env);
+  ADD_FOLDING_PRIM("keyword<?",                keyword_lt,                       1, -1, 1, env);
   
   p = scheme_make_folding_prim(string_to_keyword_prim, "string->keyword", 1, 1, 1);
   SCHEME_PRIM_PROC_FLAGS(p) |= scheme_intern_prim_opt_flags(SCHEME_PRIM_AD_HOC_OPT);
